///****************************************************************************
// File Name            : avalon2wb.v
//-----------------------------------------------------------------------------
// Function             : avalon to Wishbone BUS 
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo 
//-----------------------------------------------------------------------------
// History
// -.-- 2013/02/21
//*****************************************************************************
module avalon2wb (
csi_clockreset_clk,    // clock interface
csi_clockreset_reset_n,// reset clock interface
avs_s1_address,        // s1 slave interface
avs_s1_byteenable,     // s1 slave interface
avs_s1_read,           // s1 slave interface
avs_s1_write,          // s1 slave interface
avs_s1_writedata,      // s1 slave interface
avs_s1_readdata,       // s1 slave interface
wb_clk_o, wb_rst_o, wb_cyc_o, wb_stb_o, wb_adr_o, wb_sel_o, wb_we_o, wb_dat_o, wb_dat_i, wb_ack_i //wb 
                  
);
input csi_clockreset_clk;
input csi_clockreset_reset_n;
input [27:0] avs_s1_address;
input [3:0] avs_s1_byteenable;   
input avs_s1_read;
input avs_s1_write;
input [31:0] avs_s1_writedata;
output [31:0] avs_s1_readdata;
//
output         wb_clk_o;         // clock
output         wb_rst_o;         // reset (asynchronous active low)
output         wb_cyc_o;         // cycle
output         wb_stb_o;         // strobe
output  [29:0]  wb_adr_o;         // address adr_o
output  [3:0]  wb_sel_o;         // byteenable
output         wb_we_o;          // write enable
output  [ 31:0] wb_dat_o;         // data output
input [31:0] wb_dat_i;         // data input
input        wb_ack_i;         // normal bus termination
//
wire [1:0] byte_adr_b01;
reg [31:0] avs_s1_readdata;
//
reg         wb_cyc_o;         // cycle
reg         wb_stb_o;         // strobe
reg  [29:0]  wb_adr_o;         // address adr_o
reg  [3:0]  wb_sel_o;         // address adr_o
reg         wb_we_o;          // write enable
reg  [ 31:0] wb_dat_o;         // data output
//
assign    wb_clk_o = csi_clockreset_clk;
assign    wb_rst_o = ~csi_clockreset_reset_n;
assign byte_adr_b01 = (avs_s1_byteenable[0]==1'b1)?2'b00:                 
                      (avs_s1_byteenable[1]==1'b1)?2'b01:
                      (avs_s1_byteenable[2]==1'b1)?2'b10:
                      (avs_s1_byteenable[3]==1'b1)?2'b11:2'b00;

        
always @ (posedge csi_clockreset_clk or negedge csi_clockreset_reset_n )
  if (csi_clockreset_reset_n==1'b0)
    wb_cyc_o <= 1'b0;
  else 
    if ((avs_s1_read==1'b1)||(avs_s1_write==1'b1))
       wb_cyc_o <=  1'b1 ;
    else if (wb_ack_i==1'b1)
       wb_cyc_o <=  1'b0 ;
    else
       wb_cyc_o <=  wb_cyc_o ;
    
always @ (posedge csi_clockreset_clk or negedge csi_clockreset_reset_n )
  if (csi_clockreset_reset_n==1'b0)
    wb_stb_o <= 1'b0;
  else 
    if ((avs_s1_read==1'b1)||(avs_s1_write==1'b1))
       wb_stb_o <=  1'b1 ;
    else if (wb_ack_i==1'b1)
       wb_stb_o <=  1'b0 ;
    else
       wb_stb_o <= wb_stb_o ;
   
always @ (posedge csi_clockreset_clk or negedge csi_clockreset_reset_n )
  if (csi_clockreset_reset_n==1'b0)
    wb_we_o <= 1'b0;
  else 
    if (avs_s1_write==1'b1)
       wb_we_o <=  1'b1 ;
    else if (wb_ack_i==1'b1)
       wb_we_o <=  1'b0 ;
    else
       wb_we_o <= wb_we_o ;

always @ (posedge csi_clockreset_clk or negedge csi_clockreset_reset_n )
  if (csi_clockreset_reset_n==1'b0)
    wb_adr_o <= 30'h00000000;
  else 
    if ((avs_s1_read==1'b1)||(avs_s1_write==1'b1))
       wb_adr_o <= {avs_s1_address[27:0],byte_adr_b01};
    else
       wb_adr_o <= wb_adr_o ;
   
always @ (posedge csi_clockreset_clk or negedge csi_clockreset_reset_n )
  if (csi_clockreset_reset_n==1'b0)
    wb_sel_o <= 4'b0000;
  else 
    if ((avs_s1_read==1'b1)||(avs_s1_write==1'b1))
       wb_sel_o <= avs_s1_byteenable;
    else
       wb_sel_o <= wb_sel_o ;

always @ (posedge csi_clockreset_clk or negedge csi_clockreset_reset_n )
  if (csi_clockreset_reset_n==1'b0)
    wb_dat_o <= 32'h00000000;
  else 
    if (avs_s1_write==1'b1)
       /*
       if(avs_s1_byteenable[0]==1'b1)
         wb_dat_o <= avs_s1_writedata;
       else if (avs_s1_byteenable[1]==1'b1)  
         wb_dat_o <= {8'h00,avs_s1_writedata[31:8]};
       else if (avs_s1_byteenable[2]==1'b1)  
         wb_dat_o <= {16'h0000,avs_s1_writedata[31:16]};
       else if (avs_s1_byteenable[3]==1'b1)  
         wb_dat_o <= {24'h0000,avs_s1_writedata[31:24]};
       else
       */
         wb_dat_o <= avs_s1_writedata;
    else
       wb_dat_o <= wb_dat_o ;
        
always @ (posedge csi_clockreset_clk or negedge csi_clockreset_reset_n )
  if (csi_clockreset_reset_n==1'b0)
    avs_s1_readdata <= 32'h00000000;
  else 
    if (wb_ack_i==1'b1)
      /*
      if(avs_s1_byteenable[0]==1'b1)
        avs_s1_readdata <= wb_dat_i;
      else if (avs_s1_byteenable[1]==1'b1)  
        avs_s1_readdata <= {wb_dat_i[23:0],8'h00};
      else if (avs_s1_byteenable[2]==1'b1)  
        avs_s1_readdata <= {wb_dat_i[15:0],16'h00};
      else if (avs_s1_byteenable[3]==1'b1)  
        avs_s1_readdata <= {wb_dat_i[7:0],24'h00};
      else
      */
      avs_s1_readdata <= wb_dat_i;
    else
      avs_s1_readdata <= avs_s1_readdata ;

        
endmodule