----------------------------------------------------------------------
-- TITLE : DVI test (for DE0-nano)
--
--     VERFASSER : S.OSAFUNE (J-7SYSTEM Works)
--     DATUM     : 2012/09/04 -> 2012/09/04 (HERSTELLUNG)
--               : 2013/01/31 (FESTSTELLUNG)
----------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity dvi_block is
	generic(
		RESET_LEVEL		: std_logic := '1';	-- Positive logic reset
--		RESOLUTION		: string := "VGA"	-- 25.175MHz
		RESOLUTION		: string := "SVGA"	-- 40.000MHz
--		RESOLUTION		: string := "XGA"	-- 65.000MHz
--		RESOLUTION		: string := "ERRORTEST"
	);
	port (
		reset_n			: in  std_logic;
		dvi_de		: in  std_logic;
		dvi_blu		: in  std_logic_vector(7 downto 0);
		dvi_grn		: in  std_logic_vector(7 downto 0);
		dvi_red		: in  std_logic_vector(7 downto 0);
		dvi_hsync	: in  std_logic;
		dvi_vsync	: in  std_logic;
        videoclk_sig : in  std_logic;
		data0_p		: out std_logic;		-- GPIO1_D16
		data0_n		: out std_logic;		-- GPIO1_D17
		data1_p		: out std_logic;		-- GPIO1_D20
		data1_n		: out std_logic;		-- GPIO1_D21
		data2_p		: out std_logic;		-- GPIO1_D13
		data2_n		: out std_logic;		-- GPIO1_D12
		clock_p		: out std_logic;		-- GPIO1_D9
		clock_n		: out std_logic			-- GPIO1_D8
	);
end dvi_block;

architecture RTL of dvi_block is
	signal locked_sig		: std_logic;
	signal reset_sig		: std_logic;

	component dvi_tx_pdiff
	generic(
		RESET_LEVEL		: std_logic := '1';	-- Positive logic reset
		RESOLUTION		: string := "SVGA"	-- 40MHz
	);
	port(
		reset		: in  std_logic;
		clk			: in  std_logic;		-- Rise edge drive clock

		test_data0	: out std_logic_vector(9 downto 0);
		test_data1	: out std_logic_vector(9 downto 0);
		test_data2	: out std_logic_vector(9 downto 0);
		test_locked	: out std_logic;

		dvi_de		: in  std_logic;
		dvi_blu		: in  std_logic_vector(7 downto 0);
		dvi_grn		: in  std_logic_vector(7 downto 0);
		dvi_red		: in  std_logic_vector(7 downto 0);
		dvi_hsync	: in  std_logic;
		dvi_vsync	: in  std_logic;
		dvi_ctl		: in  std_logic_vector(3 downto 0) :="0000";

		data0_p		: out std_logic;
		data0_n		: out std_logic;
		data1_p		: out std_logic;
		data1_n		: out std_logic;
		data2_p		: out std_logic;
		data2_n		: out std_logic;
		clock_p		: out std_logic;
		clock_n		: out std_logic
	);
	end component;

begin
  
reset_sig <= not reset_n;
  
	U1 : dvi_tx_pdiff
	generic map (
		RESOLUTION	=> RESOLUTION
	)
	port map (
		reset		=> reset_sig,
		clk			=> videoclk_sig,
		test_locked	=> locked_sig,

		dvi_de		=> dvi_de,
		dvi_blu		=> dvi_blu	,
		dvi_grn		=> dvi_grn,
		dvi_red		=> dvi_red	,
		dvi_hsync	=> dvi_hsync,
		dvi_vsync	=> dvi_vsync,
		dvi_ctl		=> "0000",

		data0_p		=> data0_p,
		data0_n		=> data0_n,
		data1_p		=> data1_p,
		data1_n		=> data1_n,
		data2_p		=> data2_p,
		data2_n		=> data2_n,
		clock_p		=> clock_p,
		clock_n		=> clock_n
	);


end RTL;



----------------------------------------------------------------------
--   Copyright (C)2010-2013 J-7SYSTEM Works.  All rights Reserved.  --
----------------------------------------------------------------------
