//*****************************************************************************
// File Name            : nios2_vga_ap.v
//-----------------------------------------------------------------------------
// Function             : camera sample application
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo 
//-----------------------------------------------------------------------------
// History
// -.-- 2013/02/19
//*****************************************************************************
module nios2_vga_ap   ( clk_50m, resetb, 
                   UART_Tx,UART_Rx,
                   led,
                   hsync, vsync, blank, red_da, green_da, blue_da,  
                   ram_clk,ram_clke,ram_cs_n,ram_cas_n,    
                   ram_ras_n,ram_we_n,ram_dqml,ram_dqmu,ram_ba0,    
                   ram_ba1,ram_adr,ram_dq_l,ram_dq_u,out_en,  
                   dvi_data0_p,dvi_data0_n,dvi_data1_p,dvi_data1_n,
				   dvi_data2_p,dvi_data2_n,dvi_clock_p,dvi_clock_n,			   
                   mon
                        
);
  //
  parameter     LINE_FIFO_AWIDTH = 7;
  //whishbone_baseaddr = 0x40000000
  parameter vram_offset =   2'h0 ;      //whishbone_baseaddr + 0x
  parameter sgpio_offset =  18'h10000 ; //whishbone_baseaddr + 0x10000000
  parameter vga_offset =    18'h10001 ; //whishbone_baseaddr + 0x10001000
  //
  input clk_50m;    //clk 50MHz
  input resetb;     //reset_b
  //UART
  output UART_Tx;       //serial tx data   115200Kbps
  input  UART_Rx;       //serial rx data   115200Kbps
  //LED
  output [7:0] led;
  //vga
  output        hsync;               // horizontal sync
  output        vsync;               // vertical sync
  output        blank;               // blanking signal
  output [ 3:0] red_da, green_da, blue_da; // RGB color signals
  //dvi
  output dvi_data0_p;
  output dvi_data0_n;
  output dvi_data1_p;
  output dvi_data1_n;
  output dvi_data2_p;
  output dvi_data2_n;
  output dvi_clock_p;
  output dvi_clock_n;
  //sdram
  output ram_clk;   //adram clk 150MHz
  output ram_clke;  //sdram clke
  output ram_cs_n;  //sdram cs
  output ram_cas_n; //sdram cas
  output ram_ras_n; //sdram ras
  output ram_we_n;  //sdram write enable
  output ram_dqml;  //sdram data output enable dq[7:0]
  output ram_dqmu;  //sdram data output enable dq[15:8]
  output ram_ba0;   //sdram bank address 0
  output ram_ba1;   //sdram bank address 1
  output [11:0] ram_adr; //sdram address 
  inout [7:0] ram_dq_l; //sdram data 
  inout [7:0] ram_dq_u; //sdram data 
  output      out_en;
  //montor
  output mon;
  //
  wire  clk_50m_i;  // 50MHz 
  wire  clk_100m_i; //100MHz CLk
  wire  locked;
  wire  reset_loacked;  
  wire  resetb_loacked;   
  //wishboen bus  nios2 -> IP
  wire         clk_i;         // clock
  wire         rst_i;         // reset 1:reset
  wire         rstb_i;        // reset 0:reset
  wire         cyc_i;         // cycle
  wire         cyc_i_sgpio;         // cycle
  wire         cyc_i_vga;         // cycle
  wire         cyc_i_vram;         // cycle
  wire         stb_i;         // strobe
  wire         stb_i_sgpio;         // strobe
  wire         stb_i_vga;         // strobe
  wire         stb_i_vram;         // strobe
  wire  [29:0]  adr_i;         // address adr_i[1]
  wire  [3:0]   sel_i;         // address adr_i[1]
  wire         we_i;          // write enable
  wire         we_i_sgpio;          // write enable
  wire         we_i_vga;          // write enable
  wire         we_i_vram;          // write enable
  wire  [ 31:0] dat_i;         // data output
  wire  [ 31:0] dat_o;         // data input
  wire  [ 31:0] dat_o_vga;         // data input
  wire  [ 7:0] dat_o_sgpio;         // data input
  wire        ack_o;         // normal bus termination
  wire        ack_o_sgpio;         // normal bus termination
  wire        ack_o_vga;         // normal bus termination
  wire        ack_o_vram;         // normal bus termination
  wire        int;
  wire        rty_o;
  wire        err_o_vga;
  wire        ack_err_o_vga;
  wire        sgpio_sel;
  wire        vga_sel;
  wire        vram_sel;
  //wishboen bus vga_enh_top -> vram  
  wire    [31:0] wb_addr_o;
  wire    [31:0] wb_data_i;
  wire    [31:0] wb_data_o;
  wire    [3:0]  wb_sel_o;
  wire           wb_we_o;
  wire           wb_stb_o;
  wire           wb_cyc_o;
  wire    [2:0]  wb_cti_o;
  wire    [1:0]  wb_bte_o;
  wire           wb_ack_i;
  wire           wb_err_i=1'b0;
  wire           wb_ack_o_vram;
  //nios2 gpio
  wire  [31:0]   gpo;
  wire  [31:0]   gpi;
  //opencores simple_gpio  
  wire  [31:0]   sgpio;
  //VGA
  wire           pclk_i;     // 40MHz
  reg            hsync;               // horizontal sync
  reg            vsync;               // vertical sync
  wire           hsync_w;               // horizontal sync
  wire           vsync_w;               // vertical sync 
  wire           csync;               // composite sync
  reg            blank;               // blanking signal
  wire           blank_w;               // blanking signal
  wire           de_w;               // blanking signal
  wire   [ 7:0]  red, green, blue; // RGB color signals
  reg   [ 3:0]   red_da, green_da, blue_da; // RGB color signals
  //sdram
  wire [15:0] ram_dq_r; //sdram data 
  wire [15:0] ram_dq_w; //sdram data 
  wire [3:0] stffset =   2'h0 ;      //whishbone_baseaddr + 0x
  //    
  //reset 
  assign reset = ~resetb;
  assign reset_locked =((resetb==1'b0)||(locked==1'b0))?1'b1:1'b0;
  assign resetb_locked = ~reset_locked ;
  assign rstb_i = ~rst_i ;
  //de
  assign de_w = ~blank_w;               // blanking signal  
  //gpio                  
  assign gpi = gpo;
   
  //led
  assign led = {4'b0000,sgpio[1:0],gpo[1:0]};

  //Wishbone bus nios2 -> IP
  //select module    
  assign sgpio_sel=(adr_i[29:12]== sgpio_offset)?1'b1:1'b0;
  assign vga_sel  =(adr_i[29:12]== vga_offset)?  1'b1:1'b0;
  assign vram_sel =(adr_i[29:28]== vram_offset)? 1'b1:1'b0; //baseaddr = 0x40000000
  //read data
  assign dat_o = (sgpio_sel==1'b1)?{16'h0000,dat_o_sgpio,dat_o_sgpio}:
                 (vga_sel==1'b1)  ?dat_o_vga:
                                  32'h00000000 ;  

  //ack 
  assign ack_o = (sgpio_sel==1'b1)?ack_o_sgpio:
                 (vga_sel==1'b1)  ?ack_err_o_vga:
                 (vram_sel==1'b1) ?ack_o_vram:
                                  stb_i;
 
  assign ack_err_o_vga = ((ack_o_vga==1'b1)||(err_o_vga==1'b1))?1'b1:1'b0;
    
  assign stb_i_sgpio = (sgpio_sel==1'b1)?stb_i:1'b0;
  assign stb_i_vga =  (vga_sel==1'b1)?stb_i:1'b0;
  assign stb_i_vram = (vram_sel==1'b1)?stb_i:1'b0;
  assign cyc_i_sgpio = (sgpio_sel==1'b1)?cyc_i:1'b0;
  assign cyc_i_vga =  (vga_sel==1'b1)?cyc_i:1'b0;
  assign cyc_i_vram = (vram_sel==1'b1)?cyc_i:1'b0;
  assign we_i_sgpio =  (sgpio_sel==1'b1)?we_i:1'b0;
  assign we_i_vga =  (vga_sel==1'b1)?we_i:1'b0;
  assign we_i_vram = (vram_sel==1'b1)?we_i:1'b0;

  //Wishbone bus vga_enh_top <- vram
  assign wb_ack_i = wb_ack_o_vram;
   
  //vga
  /* 
  assign red_da = (blank==1'b1)?4'b0000:red[7:4];
  assign green_da = (blank==1'b1)?4'b0000:green[7:4];
  assign blue_da = (blank==1'b1)?4'b0000:blue[7:4];
  */

  always @ (posedge pclk_i )
    begin
       vsync <= vsync_w;
       hsync <= hsync_w;
       blank <= blank_w;
       if (blank_w==1'b0) begin
         red_da <= red[7:4];
         green_da <= green[7:4];
         blue_da <= blue[7:4];
       end
       else begin
         red_da <= 4'h0;
         green_da <= 4'h0;
         blue_da <= 4'h0;
       end        
    end // always @ (posedge pclk_i )

  //sdram
  assign ram_dq_l = (out_en==1'b1)?ram_dq_w[7:0]:8'bzzzzzzzz;
  assign ram_dq_u = (out_en==1'b1)?ram_dq_w[15:8]:8'bzzzzzzzz;
  assign ram_dq_r = {ram_dq_u,ram_dq_l};

  //mon
  assign mon =  1'b0;
   
clk_gen clk_gen(
   .areset(reset),   
   .inclk0(clk_50m),
   .c0(clk_50m_i),   // 50MHz
   .c1(clk_100m_i),  //100MHz
   .c2(pclk_i),      // 40MHz
   .c3(ram_clk),     //150MHz sdram clk
   .locked(locked));
   
nios2e_sys nios2e_sys(
   .reset_reset_n(resetb_locked),
   .clk_clk(clk_100m_i),
   .uart_0_external_connection_txd(UART_Tx),    
   .uart_0_external_connection_rxd(UART_Rx),
   .pio_0_external_connection_export(gpo),
   .pio_1_external_connection_export(gpi),
   .wb_cyc_o(cyc_i), //cyc_o
   .wb_ack_i(ack_o), //ack_i
   .wb_stb_o(stb_i), //stb_o
   .wb_adr_o(adr_i), //adr_o
   .wb_sel_o(sel_i), //sel_o
   .wb_we_o (we_i ), //we_o
   .wb_dat_o(dat_i), //dat_o
   .wb_dat_i(dat_o), //dat_i
   .wb_clk_o(clk_i), //clk_o
   .wb_rst_o(rst_i)  //rst_o
        );

//opencores simple_gpio   
 simple_gpio simple_gpio(
  .clk_i(clk_i),         // clock
  .rst_i(rstb_i),         // reset (asynchronous active low)
  .cyc_i(cyc_i_sgpio),         // cycle
  .stb_i(stb_i_sgpio),         // strobe
  .adr_i(adr_i[0]),         // address adr_i[1]
  .we_i(we_i_sgpio),          // write enable
  .dat_i(dat_i[7:0]),         // data output
  .dat_o(dat_o_sgpio),         // data input
  .ack_o(ack_o_sgpio),         // normal bus termination
  .gpio(sgpio)
                         );
 
vga_enh_top #(1'b0, LINE_FIFO_AWIDTH) u0 (
                .wb_clk_i     ( clk_i             ),
                .wb_rst_i     ( rst_i          ), // reset (active Hi)
                .rst_i        ( resetb_locked   ),// async reset (asynchronous active low)
                .wb_inta_o    ( int             ),

                //-- slave signals
                .wbs_adr_i    ( adr_i[11:0] ),
                .wbs_dat_i    ( dat_i       ),
                .wbs_dat_o    ( dat_o_vga    ),
                .wbs_sel_i    ( sel_i        ), //
                .wbs_we_i     ( we_i_vga         ),
                .wbs_stb_i    ( stb_i_vga        ),
                .wbs_cyc_i    ( cyc_i_vga        ),
                .wbs_ack_o    ( ack_o_vga    ),
                .wbs_rty_o    ( rty_o        ), //
                .wbs_err_o    ( err_o_vga    ), //

                //-- master signals
                .wbm_adr_o    ( wb_addr_o[31:0] ),
                .wbm_dat_i    ( wb_data_i      ),
                .wbm_sel_o    ( wb_sel_o        ),
                .wbm_we_o     ( wb_we_o         ),
                .wbm_stb_o    ( wb_stb_o        ),
                .wbm_cyc_o    ( wb_cyc_o        ),
                .wbm_cti_o    ( wb_cti_o        ),
                .wbm_bte_o    ( wb_bte_o        ),
                .wbm_ack_i    ( wb_ack_i        ),
                .wbm_err_i    ( wb_err_i        ),

                //-- VGA signals 
                .clk_p_i      ( pclk_i          ),
                                         /*
                .dvi_hsync_o  ( ihsync          ),
                .dvi_vsync_o  ( ivsync          ),
                .dvi_de_o     ( dvi_de_o        ),
                .dvi_d_o      ( dvi_d_o         ),
                                         */
                .clk_p_o      ( clk_p           ),
                .hsync_pad_o  ( hsync_w           ),
                .vsync_pad_o  ( vsync_w           ),
                .csync_pad_o  ( csync           ),
                .blank_pad_o  ( blank_w           ),
                .r_pad_o      ( red             ),
                .g_pad_o      ( green           ),
                .b_pad_o      ( blue            )
        );
	   
dvi_block  dvi_block (
                .reset_n        ( resetb   ),// async reset (asynchronous active low)
                //-- VGA signals 
                .videoclk_sig   ( pclk_i          ),
                .dvi_hsync  ( hsync_w           ),
                .dvi_vsync  ( vsync_w           ),
                .dvi_de  ( de_w           ),
                .dvi_red      ( red             ),
                .dvi_grn      ( green           ),
                .dvi_blu      ( blue            ),
				//DVI
				.data0_p    (dvi_data0_p),
				.data0_n    (dvi_data0_n),
				.data1_p    (dvi_data1_p),
				.data1_n    (dvi_data1_n),
				.data2_p    (dvi_data2_p),
				.data2_n    (dvi_data2_n),
				.clock_p    (dvi_clock_p),
				.clock_n    (dvi_clock_n)
        );          
   
		
//vram controller
wb_sdram_ctrl wb_sdram_ctrl(
   //Wishbone BUS
  .clk_i(clk_i),         // clock
  .rst_i(rstb_i),         // reset (asynchronous active low)
  .w_cyc_i(cyc_i_vram),         // cycle
  .w_stb_i(stb_i_vram),         // strobe
  .w_adr_i(adr_i[27:0]),         // address adr_i[1]
  .w_we_i(we_i_vram),          // write enable
  .w_dat_i(dat_i),         // 
  .w_ack_o(ack_o_vram),         // normal bus termination
  .r_cyc_i(wb_cyc_o),         // cycle
  .r_stb_i(wb_stb_o),         // strobe
  .r_adr_i(wb_addr_o[27:0]),         // address adr_i[1]
  .r_we_i(wb_we_o),          // write enable
  .r_dat_o(wb_data_i ),         // data input
  .r_ack_o(wb_ack_o_vram),         // normal bus termination
  .out_en(out_en),
  //sdrm if
  .clk(      ram_clk),            
  .reset_n(  resetb_locked),   
  .ram_clke( ram_clke),   
  .ram_cs_n( ram_cs_n),  
  .ram_cas_n(ram_cas_n),    
  .ram_ras_n(ram_ras_n), 
  .ram_we_n( ram_we_n),  
  .ram_dqml( ram_dqml),  
  .ram_dqmu( ram_dqmu),  
  .ram_ba0(  ram_ba0),
  .ram_ba1(  ram_ba1),   
  .ram_adr(  ram_adr),   
  .ram_dq_w( ram_dq_w),  
  .ram_dq_r( ram_dq_r),
  .state(    state)      
);

   
endmodule




