//*****************************************************************************
// File Name            : wb_sdram_ctrl.v
//-----------------------------------------------------------------------------
// Function             : wishbone bus sdram controller
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo@himwari_co 
//-----------------------------------------------------------------------------
// History
// -.-- 2013/02/21
//*****************************************************************************
module wb_sdram_ctrl (
  clk,
  reset_n, 
  ram_clke,    
  ram_cs_n, 
  ram_cas_n,    
  ram_ras_n,    
  ram_we_n,    
  ram_dqml,    
  ram_dqmu,    
  ram_ba0,    
  ram_ba1,
  ram_adr,
  ram_dq_w,
  ram_dq_r,
 //wishbone                
  clk_i, 
  rst_i, 
  w_cyc_i, 
  w_stb_i, 
  w_adr_i, 
  w_we_i, 
  w_dat_i, 
  w_ack_o,
  r_cyc_i, 
  r_stb_i, 
  r_adr_i,
  r_we_i,  
  r_dat_o, 
  r_ack_o,
  out_en,
  state
);

  input clk;
  input reset_n;
  output ram_clke;  //sdram clke
  output ram_cs_n;  //sdram cs
  output ram_cas_n; //sdram cas
  output ram_ras_n; //sdram ras
  output ram_we_n;  //sdram write enable
  output ram_dqml;  //sdram data output enable dq[7:0]
  output ram_dqmu;  //sdram data output enable dq[15:8]
  output ram_ba0;   //sdram bank address 0
  output ram_ba1;   //sdram bank address 1
  output [11:0] ram_adr; //sdram address 
  output [15:0] ram_dq_w; //sdram write data 
  input  [15:0] ram_dq_r; //sdram read data
  output        out_en; 
  output [3:0]   state;
  //wishbone
  input         clk_i;         // clock
  input         rst_i;         // reset (asynchronous active low)
  //write
  input         w_cyc_i;         // cycle
  input         w_stb_i;         // strobe
  input  [27:0] w_adr_i;         // address adr_i[1]
  input         w_we_i;          // write enable
  input  [ 31:0] w_dat_i;         // data output
  output        w_ack_o;         // normal bus termination

  //read
  input         r_cyc_i;         // cycle
  input         r_stb_i;         // strobe
  input  [27:0] r_adr_i;         // address adr_i[1]
  input         r_we_i;          // write enable
  output [ 31:0] r_dat_o;         // data input
  output        r_ack_o;         // normal bus termination

  parameter  p_init = 4'b0;   
  parameter  p_idle = 4'b1;   
  parameter  p_dsl  = 4'd2;   
  parameter  p_nop  = 4'd3;   
  parameter  p_pall = 4'd4;   
  parameter  p_init_ref1  = 4'd5;
  parameter  p_init_ref2  = 4'd6;
  parameter  p_mrs  = 4'd7;
  parameter  p_rd_act  = 4'd8;
  parameter  p_rd1  = 4'd9;
  //parameter  p_rd_i  = 4'd10;
  //parameter  p_rd2  = 4'd11;
  parameter  p_wr_act  = 4'd10;
  parameter  p_wr1  = 4'd11;
  //parameter  p_wr_i  = 4'd13;
  //parameter  p_wr2  = 4'd14;
  parameter  p_ref  = 4'd12;

   
  parameter  t_clear = 16'd0;
  parameter  t_inc   = 16'd1;
  //parameter  t_init_end = 16'd35000;
  parameter  t_init_end = 16'd35000;
  parameter  t_pall_end = 16'd5;
  parameter  t_ref_end = 16'd15;
  //parameter  t_ref_end = 16'd60;
  parameter  t_mrs_end = 16'd2;
  parameter  t_wr_act_end = 16'd2;
  parameter  t_rd_act_end = 16'd2;
  //parameter  t_rd1_end = 16'd12;
  //parameter  t_rd1_end = 16'd15;
  //parameter  t_rd1_end = 16'd8;//bl=2
  parameter  t_rd1_end = 16'd15;//bl=8
  //parameter  t_rd_i_end = 16'd2;
  //parameter  t_rd2_end = 16'd12;
  //parameter  t_wr1_end = 16'd7;
  //parameter  t_wr1_end = 16'd10; //bl=2
  parameter  t_wr1_end = 16'd18; //bl=8
  //parameter  t_wr_i_end = 16'd2;
  //parameter  t_wr2_end = 16'd7;
  parameter  t_ref_time = 16'd10000;
  //parameter  t_ref_time = 16'd100;
  //burst len=8 
  parameter  t_rd_da_en = 16'd6;
  parameter  t_rd_r_end = 16'd8;
  parameter  t_rd_da_en_on = 16'd6;
  parameter  t_rd_da_en_off = 16'd13;
  parameter  t_wr_dqm_start = 16'd0;
  parameter  t_wr_dqm_end = 16'd7;
  parameter  t_rd_dqm_start = 16'd1;
  parameter  t_rd_dqm_end = 16'd12;
  //
  /*
  //burst len=2
  parameter  t_rd_da_en = 16'd6;
  parameter  t_wr_dqm_start = 16'd0;
  parameter  t_wr_dqm_end = 16'd1;
  parameter  t_rd_dqm_start = 16'd1;
  parameter  t_rd_dqm_end = 16'd6;
  */
 
  reg [3:0]   state;
  reg [15:0]  waite_cnt;
  reg [15:0]  ref_cnt;
 
  reg ram_clke;  //sdram clke
  reg ram_cs_n;  //sdram cs
  reg ram_cas_n; //sdram cas
  reg ram_ras_n; //sdram ras
  reg ram_we_n;  //sdram write enable
  reg ram_dqml;  //sdram data enable dq[7:0]
  reg ram_dqmu;  //sdram data enable dq[15:8]
  reg ram_ba0;   //sdram bank address 0
  reg ram_ba1;   //sdram bank address 1
  reg [11:0] ram_adr; //sdram address 
  reg [15:0] ram_dq_w; //sdram write data 
  reg [15:0] ram_dq_r_d1; //sdram read data 1clk delay 
  reg  out_en;
  reg [31:0]r_data_0; //read  data
  reg [31:0]r_data_1; //read  data
  reg [31:0]r_data_2; //read  data
  reg [31:0]r_data_3; //read  data
  reg  w_end;
  reg  r_end;  
   
  reg [31:0] w_data_0;
  reg [31:0] w_data_1;
  reg [31:0] w_data_2;
  reg [31:0] w_data_3;
  reg [19:0] w_adr;
  reg  w_req;
  reg  w_ack_o;
 
  reg [19:0] r_adr;
  reg [31:0]r_dat_o; //user read  data
  reg  r_req;
  reg  r_ack_o;
  reg  [1:0] r_dat_cnt;

   
//

always @ (posedge clk_i or negedge rst_i )
if (rst_i==1'b0)
  begin
    w_adr <= 20'h00000;
    w_data_0 <= 32'h0000000;
    w_data_1 <= 32'h0000000;
    w_data_2 <= 32'h0000000;
    w_data_3 <= 32'h0000000;
  end
else
  if (w_stb_i==1'b1)
    begin
      w_adr <= {w_adr_i[19:4],4'b0000};
      if (w_adr_i[3:2]==2'b00)
         w_data_0 <= w_dat_i;
      else
         w_data_0 <= w_data_0;  
      if (w_adr_i[3:2]==2'b01)
         w_data_1 <= w_dat_i;
      else
         w_data_1 <= w_data_1;  
      if (w_adr_i[3:2]==2'b10)
         w_data_2 <= w_dat_i;
      else
         w_data_2 <= w_data_2;  
      if (w_adr_i[3:2]==2'b11)
         w_data_3 <= w_dat_i;
      else
         w_data_3 <= w_data_3;  
    end
  else
    begin
      w_adr <= w_adr;
      w_data_0 <= w_data_0;
      w_data_1 <= w_data_1;
      w_data_2 <= w_data_2;
      w_data_3 <= w_data_3;
    end


always @ (posedge clk_i or negedge rst_i )
if (rst_i==1'b0)
  w_req <= 1'b0;
else
  if ((w_stb_i==1'b1)&&(w_cyc_i==1'b1)&&(w_we_i==1'b1)&&(w_req==1'b0)&&(w_end==1'b0)&&(w_adr_i[3:2]==2'b11))
    w_req <= 1'b1;
  else if ((w_req==1'b1)&&(w_end==1'b1))
    w_req <= 1'b0;
  else
    w_req <= w_req;


always @ (posedge clk_i or negedge rst_i )
if (rst_i==1'b0)
  w_ack_o <= 1'b0;
else
  if ((w_ack_o==1'b0)&&(w_stb_i==1'b1)&&(w_cyc_i==1'b1))
    w_ack_o <= 1'b1;
  else
    w_ack_o <= 1'b0;
    
always @ (posedge clk_i or negedge rst_i )
if (rst_i==1'b0)
  begin
    r_adr <= 20'h00000;
  end
else
  if ((r_stb_i==1'b1)&&(r_cyc_i==1'b1)&&(r_req==1'b0)&&(r_end==1'b0))
    begin
      r_adr <= r_adr_i[19:0];
    end
  else
    begin
      r_adr <= r_adr;
    end
     
   
always @ (posedge clk_i or negedge rst_i )
if (rst_i==1'b0)
  r_dat_cnt <= 2'd0;
else
  if( r_ack_o==1'b1)
    r_dat_cnt <= r_dat_cnt + 2'b1;
  else if(r_req==1'b1)
    r_dat_cnt <= 2'd0;
  else
    r_dat_cnt <= r_dat_cnt ;
   
always @ (posedge clk_i or negedge rst_i )
if (rst_i==1'b0)
    r_dat_o <= 32'h0000000;
else
  case(r_dat_cnt)
    2'b00: r_dat_o <= r_data_0;
    2'b01: r_dat_o <= r_data_1;
    2'b10: r_dat_o <= r_data_2;
    2'b11: r_dat_o <= r_data_3;
  endcase
      
always @ (posedge clk_i or negedge rst_i )
if (rst_i==1'b0)
  r_req <= 1'b0;
else
  if ((r_stb_i==1'b1)&&(r_cyc_i==1'b1)&&(r_req==1'b0)&&(r_end==1'b0))
    r_req <= 1'b1;
  else if ((r_req==1'b1)&&(r_ack_o==1'b1))
    r_req <= 1'b0;
  else
    r_req <= r_req;

always @ (posedge clk_i or negedge rst_i )
if (rst_i==1'b0)
  r_ack_o <= 1'b0;
else
  if ((r_req==1'b1)&&(r_end==1'b1)&&(r_dat_cnt==2'b00)&&(r_stb_i==1'b1)&&(r_cyc_i==1'b1))
    r_ack_o <= 1'b1;
  else if (r_dat_cnt==2'b11)
    r_ack_o <= 1'b0;
  else 
    r_ack_o <= r_ack_o;
   
//Xe[g}V  
always @ (posedge clk or negedge reset_n )
if (reset_n==1'b0)
  begin
    state <= p_init;
    waite_cnt <= t_clear;
  end
else
  case (state)
    //-------------------------------------------------
    p_init:begin
        if (waite_cnt==t_init_end) //210us
          begin
            state <= p_pall;
            waite_cnt <= t_clear;
          end
        else
          begin
            state <= p_init;
            waite_cnt <= waite_cnt + t_inc;
          end
      end // 
    //-------------------------------------------------
    p_pall:begin
        if (waite_cnt==t_pall_end) 
          begin
            state <= p_init_ref1;
            waite_cnt <= t_clear;
          end
        else
          begin
            state <= p_pall;
            waite_cnt <= waite_cnt + t_inc;
          end
      end // 
    //-------------------------------------------------
    p_init_ref1:begin
        if (waite_cnt==t_ref_end) 
          begin
            state <= p_init_ref2;
            waite_cnt <= t_clear;
          end
        else
          begin
            state <= p_init_ref1;
            waite_cnt <= waite_cnt + t_inc;
          end
      end // 
    //-------------------------------------------------
    p_init_ref2:begin
        if (waite_cnt==t_ref_end) 
          begin
            state <= p_mrs;
            waite_cnt <= t_clear;
          end
        else
          begin
            state <= p_init_ref2;
            waite_cnt <= waite_cnt + t_inc;
          end
      end // 
    //-------------------------------------------------
    p_mrs:begin
        if (waite_cnt==t_mrs_end) 
          begin
            state <= p_nop;
            waite_cnt <= t_clear;
          end
        else
          begin
            state <= p_mrs;
            waite_cnt <= waite_cnt + t_inc;
          end
      end // 
    //-------------------------------------------------
    p_nop:begin
      if  ((r_req==1'b1)&&(r_end==1'b0)) 
        state <= p_rd_act;
      else if (w_req==1'b1)
        state <= p_wr_act;
      else if (ref_cnt >= t_ref_time)
        state <= p_ref;
      else
        state <= p_nop;
      waite_cnt <= t_clear;
      end // 
    //-------------------------------------------------
    p_rd_act:begin
      if (waite_cnt==t_rd_act_end)
        begin
          state <= p_rd1;
          waite_cnt <= t_clear;
        end
      else
        begin
          state <= p_rd_act;
          waite_cnt <= waite_cnt + t_inc;
        end
      end // 
    //-------------------------------------------------
    p_rd1:begin
      if (waite_cnt==t_rd1_end)
        begin
          //state <= p_rd_i;
          state <= p_nop;
          waite_cnt <= t_clear;
        end
      else
        begin
          state <= p_rd1;
          waite_cnt <= waite_cnt + t_inc;
        end
      end // 
    //-------------------------------------------------
    /*
    p_rd_i:begin
      if (waite_cnt==t_rd_i_end)
        begin
          state <= p_rd2;
          waite_cnt <= t_clear;
        end
      else
        begin
          state <= p_rd_i;
          waite_cnt <= waite_cnt + t_inc;
        end
      end // 
    //-------------------------------------------------
    p_rd2:begin
      if (waite_cnt==t_rd2_end)
        begin
          state <= p_nop;
          waite_cnt <= t_clear;
        end
      else
        begin
          state <= p_rd2;
          waite_cnt <= waite_cnt + t_inc;
        end
      end // 
    */
    //-------------------------------------------------
    p_wr_act:begin
      if (waite_cnt==t_wr_act_end)
        begin
          state <= p_wr1;
          waite_cnt <= t_clear;
        end
      else
        begin
          state <= p_wr_act;
           waite_cnt <= waite_cnt + t_inc;
        end
      end // 
    //-------------------------------------------------
    p_wr1:begin
      if (waite_cnt==t_wr1_end)
        begin
          //state <= p_wr_i;
          state <= p_nop;
          waite_cnt <= t_clear;
        end
      else
        begin
          state <= p_wr1;
          waite_cnt <= waite_cnt + t_inc;
        end
      end //
    /*
    //-------------------------------------------------
    p_wr_i:begin
      if (waite_cnt==t_wr_i_end)
        begin
          state <= p_wr2;
          waite_cnt <= t_clear;
        end
      else
        begin
          state <= p_wr_i;
          waite_cnt <= waite_cnt + t_inc;
        end
      end // 
    //-------------------------------------------------
    p_wr2:begin
      if (waite_cnt==t_wr2_end)
        begin
          state <= p_nop;
          waite_cnt <= t_clear;
        end
      else
        begin
          state <= p_wr2;
          waite_cnt <= waite_cnt + t_inc;
        end
      end // 
    */
    //-------------------------------------------------
    p_ref:begin
        if (waite_cnt==t_ref_end) 
          begin
            state <= p_nop;
            waite_cnt <= t_clear;
          end
        else
          begin
            state <= p_ref;
            waite_cnt <= waite_cnt + t_inc;
          end
      end // 
  endcase


   
always @ (posedge clk or negedge reset_n )
if (reset_n==1'b0)
  ref_cnt <= 16'h0000;
else
  if (state==p_ref)
    ref_cnt <= 16'h0000;
  else
    ref_cnt <= ref_cnt + 16'h0001;


// sdram signels
   
always @ (posedge clk or negedge reset_n )
if (reset_n==1'b0)
  begin
    ram_clke <= 1'b1;  //sdram clke
    ram_cs_n <= 1'b1;  //sdram cs
    ram_cas_n <= 1'b0; //sdram cas
    ram_ras_n <= 1'b0; //sdram ras
    ram_we_n <= 1'b0;  //sdram write enable
    ram_dqml <= 1'b1;  //sdram data   enable dq[7:0]
    ram_dqmu <= 1'b1;  //sdram data   enable dq[15:8]
    ram_ba0 <= 1'b0;   //sdram bank address 0
    ram_ba1 <= 1'b0;   //sdram bank address 1
    ram_adr <= 12'b0; //sdram address 
    ram_dq_w <= 16'h0000; //sdram write data
    out_en <= 1'b0;
  end
else
  if (waite_cnt==16'h0000)
  case (state)
    //-------------------------------------------------
    p_init:
    begin
      ram_clke <= 1'b1;  
      ram_cs_n <= 1'b1;  
      ram_ras_n <= 1'b0; 
      ram_cas_n <= 1'b0; 
      ram_we_n <= 1'b0;  
      ram_dqml <= 1'b1;  
      ram_dqmu <= 1'b1;  
      ram_ba0 <= 1'b0;   
      ram_ba1 <= 1'b0;   
      ram_adr <= 12'b0;   
      ram_dq_w <= 16'h0000;  
      out_en <= 1'b0; 
    end
    //-------------------------------------------------
    p_pall:
    begin
      ram_clke <= 1'b1;  
      ram_cs_n <= 1'b0;  
      ram_ras_n <= 1'b0; 
      ram_cas_n <= 1'b1; 
      ram_we_n <= 1'b0;  
      ram_dqml <= 1'b1;  
      ram_dqmu <= 1'b1;  
      ram_ba0 <= 1'b0;   
      ram_ba1 <= 1'b0;   
      ram_adr <= 12'h400;   
      ram_dq_w <= 16'h0000;
      out_en <= 1'b0;   
    end
    //-------------------------------------------------
    p_init_ref1:
    begin
      ram_clke <= 1'b1;  
      ram_cs_n <= 1'b0;  
      ram_ras_n <= 1'b0; 
      ram_cas_n <= 1'b0; 
      ram_we_n <= 1'b1;  
      ram_dqml <= 1'b1;  
      ram_dqmu <= 1'b1;  
      ram_ba0 <= 1'b0;   
      ram_ba1 <= 1'b0;   
      ram_adr <= 12'h000;   
      ram_dq_w <= 16'h0000;
      out_en <= 1'b0;   
    end
    //-------------------------------------------------
    p_init_ref2:
    begin
      ram_clke <= 1'b1;  
      ram_cs_n <= 1'b0;  
      ram_ras_n <= 1'b0; 
      ram_cas_n <= 1'b0; 
      ram_we_n <= 1'b1;  
      ram_dqml <= 1'b1;  
      ram_dqmu <= 1'b1;  
      ram_ba0 <= 1'b0;   
      ram_ba1 <= 1'b0;   
      ram_adr <= 12'h000;   
      ram_dq_w <= 16'h0000;
      out_en <= 1'b0;   
    end
    //-------------------------------------------------
    p_mrs:
    begin
      ram_clke <= 1'b1;  
      ram_cs_n <= 1'b0;  
      ram_ras_n <= 1'b0; 
      ram_cas_n <= 1'b0; 
      ram_we_n <= 1'b0;  
      ram_dqml <= 1'b1;  
      ram_dqmu <= 1'b1;  
      ram_ba0 <= 1'b0;   
      ram_ba1 <= 1'b0;   
      //ram_adr <= 12'h31; //Burst Length=2   
      ram_adr <= 12'h33; //Burst Length=8  
      ram_dq_w <= 16'h0000;
      out_en <= 1'b0;   
    end
    //-------------------------------------------------
    p_nop:
    begin
      ram_clke <= 1'b1;  
      ram_cs_n <= 1'b1;  
      ram_ras_n <= 1'b1; 
      ram_cas_n <= 1'b1; 
      ram_we_n <= 1'b1;  
      ram_dqml <= 1'b1;  
      ram_dqmu <= 1'b1;  
      ram_ba0 <= 1'b0;   
      ram_ba1 <= 1'b0;   
      ram_adr <= 12'h000;   
      ram_dq_w <= 16'h0000;  
      out_en <= 1'b0; 
    end
    //-------------------------------------------------
    p_rd_act:
    begin
      ram_clke <= 1'b1;  
      ram_cs_n <= 1'b0;  
      ram_ras_n <= 1'b0; 
      ram_cas_n <= 1'b1; 
      ram_we_n <= 1'b1;  
      ram_dqml <= 1'b1;  
      ram_dqmu <= 1'b1;  
      ram_ba0 <= r_adr[18];   
      ram_ba1 <= r_adr[19];   
      ram_adr <= r_adr[17:8];
      ram_dq_w <= 16'h0000;
      out_en <= 1'b0;   
    end
    //-------------------------------------------------
    p_rd1:
    begin
      ram_clke <= 1'b1;  
      ram_cs_n <= 1'b0;  
      ram_ras_n <= 1'b1; 
      ram_cas_n <= 1'b0; 
      ram_we_n <= 1'b1;  
      ram_dqml <= 1'b1;  
      ram_dqmu <= 1'b1;  
      ram_ba0 <= r_adr[18];   
      ram_ba1 <= r_adr[19];   
      ram_adr <= {4'b0100,r_adr[7:0]};   
      ram_dq_w <= 16'h0000; 
      out_en <= 1'b0;   
    end // case: p_rd1
    /*
    //-------------------------------------------------
    p_rd_i:
    begin
      ram_clke <= 1'b1;  
      ram_cs_n <= 1'b1;  
      ram_ras_n <= 1'b1; 
      ram_cas_n <= 1'b1; 
      ram_we_n <= 1'b1;  
      ram_dqml <= 1'b1;  
      ram_dqmu <= 1'b1;  
      ram_ba0 <= w_adr_i[18];   
      ram_ba1 <= w_adr_i[19];   
      ram_adr <= {4'b0000,w_adr_i[7:0]};   
      ram_dq_w <= 16'h0000; 
      out_en <= 1'b0;   
    end
    //-------------------------------------------------
    p_rd2:
    begin
      ram_clke <= 1'b1;  
      ram_cs_n <= 1'b0;  
      ram_ras_n <= 1'b1; 
      ram_cas_n <= 1'b0; 
      ram_we_n <= 1'b1;  
      ram_dqml <= 1'b1;  
      ram_dqmu <= 1'b1;  
      ram_ba0 <= r_adr[18];   
      ram_ba1 <= r_adr[19];   
      ram_adr <= {4'b0100,(r_adr[7:0]+8'h8)};   
      ram_dq_w <= 16'h0000; 
      out_en <= 1'b0;   
    end
    */
    //-------------------------------------------------
    p_wr_act:
    begin
      ram_clke <= 1'b1;  
      ram_cs_n <= 1'b0;  
      ram_ras_n <= 1'b0; 
      ram_cas_n <= 1'b1; 
      ram_we_n <= 1'b1;  
      ram_dqml <= 1'b1;  
      ram_dqmu <= 1'b1;  
      ram_ba0 <= w_adr[18];   
      ram_ba1 <= w_adr[19];   
      ram_adr <= w_adr[17:8];
      ram_dq_w <= 16'h0000;
      out_en <= 1'b0;   
    end
    //-------------------------------------------------
    p_wr1:
    begin
      ram_clke <= 1'b1;  
      ram_cs_n <= 1'b0;  
      ram_ras_n <= 1'b1; 
      ram_cas_n <= 1'b0; 
      ram_we_n <= 1'b0;  
      ram_dqml <= 1'b0;  
      ram_dqmu <= 1'b0;  
      ram_ba0 <= w_adr[18];   
      ram_ba1 <= w_adr[19];   
      ram_adr <= {4'b0100,w_adr[7:0]};   
      ram_dq_w <=w_data_0[31:16]; 
      out_en <= 1'b1;    
    end // case: p_wr
    /*
    //-------------------------------------------------
    p_wr_i:
    begin
      ram_clke <= 1'b1;  
      ram_cs_n <= 1'b1;  
      ram_ras_n <= 1'b1; 
      ram_cas_n <= 1'b1; 
      ram_we_n <= 1'b1;  
      ram_dqml <= 1'b1;  
      ram_dqmu <= 1'b1;  
      ram_ba0 <= w_adr[18];   
      ram_ba1 <= w_adr[19];   
      ram_adr <= {4'b0100,w_adr[7:0]};   
      ram_dq_w <= ram_dq_w; 
      out_en <= 1'b1;    
    end // case: p_wr
    //-------------------------------------------------
    p_wr2:
    begin
      ram_clke <= 1'b1;  
      ram_cs_n <= 1'b0;  
      ram_ras_n <= 1'b1; 
      ram_cas_n <= 1'b0; 
      ram_we_n <= 1'b0;  
      ram_dqml <= 1'b0;  
      ram_dqmu <= 1'b0;  
      ram_ba0 <= w_adr[18];   
      ram_ba1 <= w_adr[19];   
      ram_adr <= {4'b0100,(w_adr[7:0]+8'h8)};   
      ram_dq_w <= ram_dq_w; 
      out_en <= 1'b1;    
    end // case: p_wr
    */
    //-------------------------------------------------
    p_ref:
    begin
      ram_clke <= 1'b1;  
      ram_cs_n <= 1'b0;  
      ram_ras_n <= 1'b0; 
      ram_cas_n <= 1'b0; 
      ram_we_n <= 1'b1;  
      ram_dqml <= 1'b1;  
      ram_dqmu <= 1'b1;  
      ram_ba0 <= 1'b0;   
      ram_ba1 <= 1'b0;   
      ram_adr <= 12'h000;   
      ram_dq_w <= 16'h0000;
      out_en <= 1'b0;     
    end
  endcase // case(state)
  else
    begin
      ram_clke <= 1'b1;  
      ram_cs_n <= 1'b1;  
      ram_ras_n <= 1'b1; 
      ram_cas_n <= 1'b1; 
      ram_we_n <= 1'b1;
      /* 
      //if ((state==p_wr1)||(state==p_wr2)||(state==p_rd1)||(state==p_rd2))
      if ((state==p_wr1)||(state==p_rd1))
      //if ((state==p_wr1)||(state==p_wr2))
        begin 
          ram_dqml <= 1'b0;  
          ram_dqmu <= 1'b0; 
          ram_dq_w <= u_wr_da;  
        end
      else
        begin 
          ram_dqml <= 1'b1;  
          ram_dqmu <= 1'b1;
          ram_dq_w <= 16'h0000;  
        end
      */
      if (((state==p_wr1)&&(waite_cnt>=t_wr_dqm_start)&&(waite_cnt<=t_wr_dqm_end))
          ||((state==p_rd1)&&(waite_cnt>=t_rd_dqm_start)&&(waite_cnt<=t_rd_dqm_end)))
        begin   
          ram_dqml <= 1'b0;   
          ram_dqmu <= 1'b0;
          case(waite_cnt)
            t_rd_dqm_start:ram_dq_w <=w_data_0[15:0];
            t_rd_dqm_start+1:ram_dq_w <=w_data_1[31:16];
            t_rd_dqm_start+2:ram_dq_w <=w_data_1[15:0];
            t_rd_dqm_start+3:ram_dq_w <=w_data_2[31:16];
            t_rd_dqm_start+4:ram_dq_w <=w_data_2[15:0];
            t_rd_dqm_start+5:ram_dq_w <=w_data_3[31:16];
            t_rd_dqm_start+6:ram_dq_w <=w_data_3[15:0];
            default:ram_dq_w <= 16'h0000;
          endcase
        end
      else
        begin 
          ram_dqml <= 1'b1;  
          ram_dqmu <= 1'b1;
          ram_dq_w <= 16'h0000;  
        end
      ram_ba0 <= 1'b0;   
      ram_ba1 <= 1'b0;   
      ram_adr <= 12'h000;
      if (state==p_wr1)
        out_en <= 1'b1;
      else
        out_en <= 1'b0;              
    end


// wishbonebus signels
   
always @ (posedge clk or negedge reset_n )
if (reset_n==1'b0)
  begin
    w_end <= 1'b0;
    r_end <= 1'b0;
  end
else
  begin
    if (state==p_nop)
      w_end <= 1'b0;
    else if ((state==p_wr1)&&(waite_cnt==16'h0000))
      w_end <= 1'b1;
    else
      w_end <= w_end;
    //   
    if (state==p_nop)
      r_end <= 1'b0;
    else if ((state==p_rd1)&&(waite_cnt==t_rd_r_end))     
      r_end <= 1'b1;
    else
      r_end <= r_end;
  end
     
      
always @ (posedge clk or negedge reset_n )
if (reset_n==1'b0)
  ram_dq_r_d1 <= 16'h0000;
else
   ram_dq_r_d1 <= ram_dq_r;
             
always @ (posedge clk or negedge reset_n )
if (reset_n==1'b0) begin
  r_data_0 <= 32'h00000000;
  r_data_1 <= 32'h00000000;
  r_data_2 <= 32'h00000000;
  r_data_3 <= 32'h00000000;
  end
else
 if ((state==p_rd1)&&(waite_cnt>=t_rd_da_en_on)&&(waite_cnt<=t_rd_da_en_off))
   begin   
      if (waite_cnt==t_rd_da_en_on)
         r_data_0 <= {ram_dq_r_d1,ram_dq_r};
      else
         r_data_0 <= r_data_0;  
      if  (waite_cnt==t_rd_da_en_on + 2)
         r_data_1 <= {ram_dq_r_d1,ram_dq_r};
      else
         r_data_1 <= r_data_1;  
      if  (waite_cnt==t_rd_da_en_on + 4)
         r_data_2 <= {ram_dq_r_d1,ram_dq_r};
      else
         r_data_2 <= r_data_2;  
      if  (waite_cnt==t_rd_da_en_on + 6)
         r_data_3 <= {ram_dq_r_d1,ram_dq_r};
      else
         r_data_3 <= r_data_3;
   end
 else
   begin
     r_data_0 <= r_data_0 ;
     r_data_1 <= r_data_1 ;
     r_data_2 <= r_data_2 ;
     r_data_3 <= r_data_3 ;
   end
  
endmodule








